#!/usr/bin/perl

use Net::FTP;


################################################################
# program : caps project backup automation script
# author : seungkwon kim (teacher.seal@gmail.com)
# company : KAIST CAIS
# version : v1.2
# first draft : 2006-10-16
# last update : 2006-10-20
################################################################

################################################################
# calculate yyyy_mm_dd pattern and prepare log file
#################################################################

my ($sec,$min,$hour,$mday,$mon,$year,$wday, $yday,$isdst)=localtime(time);
my $yyyy_mm_dd = sprintf( "%4d_%02d_%02d", $year+1900,$mon+1,$mday );
my $log_dir = "/home/backup/log/";
my $log_summary_file = "+\n+ Backup Automation Result Summary - $yyyy_mm_dd\n";
my $log_summary_smtp = "+<br>+ Backup Automation Result Summary - $yyyy_mm_dd<br>+<br>";

################################################################
# collaboration(Confluence/JIRA) backup
################################################################

my $conf_src_dir = "/home/collaboration/confluence-config/backups/";
my $conf_bak_dir = "/home/backup/collaboration/confluence/";

my $conf_src_file = $conf_src_dir . "daily-backup-" . $yyyy_mm_dd . ".zip";
my $conf_bak_file = $conf_bak_dir . "confluence-backup-" . $yyyy_mm_dd . ".zip";
if (-e $conf_src_file) {
  `cp $conf_src_file $conf_bak_file`;
   addlog("confluence backup file generation", $conf_bak_file);
} else {
   addlog("confluence backup src file doesn't exist", $conf_src_file);
}

################################################################
# svn snapshot backup 
################################################################

my $snapshot_caps_src_dir = "/home/backup/repos/caps/";
my $snapshot_caps_bak_dir = "/home/backup/snapshot/caps/";
my $snapshot_caps_bak_file = $snapshot_caps_bak_dir . "caps-snapshot-" . $yyyy_mm_dd . ".tar.gz";
my $snapshot_caps_log_file = $log_dir . "caps-snapshot-" . $yyyy_mm_dd . ".log";

`cd $snapshot_caps_src_dir`;
`svn update`;
`tar czvf $snapshot_caps_bak_file $snapshot_caps_src_dir* > $snapshot_caps_log_file`;

addlog("caps project snapshot backup file generation", $snapshot_caps_bak_file);

my $snapshot_capsadmin_src_dir = "/home/backup/repos/capsadmin/";
my $snapshot_capsadmin_bak_dir = "/home/backup/snapshot/capsadmin/";
my $snapshot_capsadmin_bak_file = $snapshot_capsadmin_bak_dir . "capsadmin-snapshot-" . $yyyy_mm_dd . ".tar.gz";
my $snapshot_capsadmin_log_file = $log_dir . "capsadmin-snapshot-" . $yyyy_mm_dd . ".log";

`cd $snapshot_capsadmin_src_dir`;
`svn update`;
`tar czvf $snapshot_capsadmin_bak_file $snapshot_capsadmin_src_dir* > $snapshot_capsadmin_log_file`;

addlog("capsadmin project snapshot backup file generation", $snapshot_capsadmin_bak_file);

################################################################
# svn dump backup - caps
################################################################

my $svndump_caps_src_dir = "/home/svn/repos/caps";
my $svndump_caps_bak_file = "/home/backup/svndump/caps/caps-svndump-" . $yyyy_mm_dd . ".dmp";
my $svndump_caps_log_file = "/home/backup/log/caps-svndump-" . $yyyy_mm_dd . ".log";

`/home/backup/script/svndump.sh $svndump_caps_src_dir $svndump_caps_bak_file $svndump_caps_log_file`;

addlog("caps project svndump backup file generation", $svndump_caps_bak_file);

################################################################
# svn dump backup - analysis
################################################################

my $svndump_analysis_src_dir = "/home/svn/repos/analysis";
my $svndump_analysis_bak_file = "/home/backup/svndump/analysis/analysis-svndump-" . $yyyy_mm_dd . ".dmp";
my $svndump_analysis_log_file = "/home/backup/log/analysis-svndump-" . $yyyy_mm_dd . ".log";

`/home/backup/script/svndump.sh $svndump_analysis_src_dir $svndump_analysis_bak_file $svndump_analysis_log_file`;

addlog("analysis team svndump backup file generation", $svndump_analysis_bak_file);

################################################################
# database backup 
################################################################

my $db_caps_bak_file = "/home/backup/database/caps/caps-db-" . $yyyy_mm_dd . ".dmp";
my $db_capsadmin_bak_file = "/home/backup/database/capsadmin/capsadmin-db-" . $yyyy_mm_dd . ".dmp";

`/home/backup/script/pgbackup.sh caps $db_caps_bak_file`;

addlog("caps database backup file generation", $db_caps_bak_file);

`/home/backup/script/pgbackup.sh capsadmin $db_capsadmin_bak_file`;

addlog("capsadmin database backup file generation", $db_capsadmin_bak_file);


################################################################
# Write Summary Log 
################################################################
ftptransfer();
printlog();
maillog();

################################################################
# Common Subroutines 
################################################################
sub addlog {
  my($message, $file) = @_;
  my $status = "";
  if (-e $file) {
    $status = "[OK] ";
  } else {
    $status = "[ERROR] ";
  }
  $log_summary_file = $log_summary_file . $status . $message . " - " . $file . "\n";
  $log_summary_smtp = $log_summary_smtp . $status . $message . " - " . $file . "<br>\n";
}

sub printlog {
  my $log_file = $log_dir . "backup-" . $yyyy_mm_dd . ".log";
  open(LOG_FILE, ">$log_file");
  print LOG_FILE $log_summary_file;
  close LOG_FILE;
}

sub maillog {
  $sendmail_path = "/usr/sbin/sendmail";
  $from_addr = "̸ּ";
  @to_addrs = (
    "뺸1",
    "뺸2",
  );
  foreach $to_addr (@to_addrs) {
    open(SENDMAIL, "| $sendmail_path $to_addr") or die "can't mail";
    print SENDMAIL "From: $from_addr\n";
    print SENDMAIL "To: $to_addr\n";
    print SENDMAIL "Subject: Backup Automation Result from CAPS - $yyyy_mm_dd\n";
    print SENDMAIL "Content-type: text/html\n\n";
    print SENDMAIL $log_summary_smtp;
    close(SENDMAIL);
  }
}

sub addftplog {
  my($newerr, $message) = @_;
  if ($newerr) {
    $log_summary_file = $log_summary_file . "[FTP Transfer Error] " . $message . "\n";
    $log_summary_smtp = $log_summary_smtp . "[FTP Transfer Error] " . $message . "<br>\n";
  } else {
    $log_summary_file = $log_summary_file . "[FTP Transfer OK] " . $messsage . "\n";
    $log_summary_smtp = $log_summary_smtp . "[FTP Transfer OK] " . $message . "<br>\n";
  }
}

sub ftptransfer {
  my $host="FTP";
  my $directory="/";

  $ftp=Net::FTP->new($host,Timeout=>240) or $connecterr=1;
  addftplog($connecterr, "FTP connection to $host");

  $ftp->login("FTP̵", "FTPйȣ") or $loginerr=1;
  addftplog($loginerr, "FTP login to $host");
  
  $ftp->binary();
  addftplog(0, "FTP set binary mode");

  my $conf_target_dir = "/backup/collaboration/confluence";
  $ftp->cwd($conf_target_dir) or $conferr=1;
  $ftp->put($conf_bak_file) or $conferr=1;
  addftplog($conferr, "FTP transfer $conf_bak_file to $conf_target_dir");

  my $snapshot_caps_target_dir = "/backup/snapshot/caps";
  $ftp->cwd($snapshot_caps_target_dir) or $sscapserr=1;
  $ftp->put($snapshot_caps_bak_file) or $sscapserr=1;
  addftplog($sscapserr, "FTP transfer $snapshot_caps_bak_file to $snapshot_caps_target_dir");

  my $snapshot_capsadmin_target_dir = "/backup/snapshot/capsadmin";
  $ftp->cwd($snapshot_capsadmin_target_dir) or $sscapsadminerr=1;
  $ftp->put($snapshot_capsadmin_bak_file) or $sscapsadminerr=1;
  addftplog($sscapsadminerr, "FTP transfer $snapshot_capsadmin_bak_file to $snapshot_capsadmin_target_dir");

  my $svndump_caps_target_dir = "/backup/svndump/caps";
  $ftp->cwd($svndump_caps_target_dir) or $svncapserr=1;
  $ftp->put($svndump_caps_bak_file) or $svncapserr=1;
  addftplog($svncapserr, "FTP transfer $svndump_caps_bak_file to $svndump_caps_target_dir");
  
  my $svndump_analysis_target_dir = "/backup/svndump/analysis";
  $ftp->cwd($svndump_analysis_target_dir) or $svnanalysiserr=1;
  $ftp->put($svndump_analysis_bak_file) or $svnanalysiserr=1;
  addftplog($svnanalysiserr, "FTP transfer $svndump_analysis_bak_file to $svndump_analysis_target_dir");
  
  my $db_caps_target_dir = "/backup/database/caps";
  $ftp->cwd($db_caps_target_dir) or $dbcapserr=1;
  $ftp->put($db_caps_bak_file) or $dbcapserr=1;
  addftplog($dbcapserr, "FTP transfer $db_caps_bak_file to $db_caps_target_dir");

  my $db_capsadmin_target_dir = "/backup/database/capsadmin";
  $ftp->cwd($db_capsadmin_target_dir) or $dbcapsadminerr=1;
  $ftp->put($db_capsadmin_bak_file) or $dbcapsadminerr=1;
  addftplog($dbcapsadminerr, "FTP transfer $db_capsadmin_bak_file to $db_capsadmin_target_dir");
  
  $ftp->quit;
}
